using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.Practices.WizardFramework;
using System.Collections;
using System.Configuration;
using Microsoft.Practices.RepositoryFactory.CustomPages.Helpers;
using Microsoft.Practices.RepositoryFactory.CustomPages.UI;
using System.ComponentModel.Design;
using System.Windows.Forms.Design;
using SD = Microsoft.Practices.RepositoryFactory.SchemaDiscovery.ObjectModel.Base;
using Microsoft.Practices.RepositoryFactory.SchemaDiscovery.ObjectModel.Base;
using Microsoft.Practices.RepositoryFactory.CustomPages.Validators;

namespace Microsoft.Practices.RepositoryFactory.CustomPages
{
    public partial class EntitiesTablesSelectorCP : CustomWizardPage
    {
        #region Fields
        private TreeView tables;
        string cnstring;
        #endregion

        #region Constructors
        public EntitiesTablesSelectorCP(WizardForm parent)
            : base(parent)
        {
            InitializeComponent();
        }
        #endregion

        #region Arguments
        [RecipeArgument]
        public TreeView TablesTreeView
        {
            get
            {
                return this.tables;
            }
            set
            {
                this.tables = value;
            }
        }
        #endregion

        #region Event Handlers
        private void trvDatabase_BeforeSelect(object sender, TreeViewCancelEventArgs e)
        {
            if(e.Node is TableNode)
            {
                Table table = (Table)e.Node.Tag;
                
                if(GetInvalidColumns(table) == table.Columns.Count)
                {
                    e.Cancel = true;
                    return;
                }
            }
            else if(e.Node is ColumnNode)
            {
                if(!new ColumnValidator((Column)e.Node.Tag).IsValid())
                {
                    e.Cancel = true;
                    return;
                }
            }
        }

        private void trvDatabase_BeforeCheck(object sender, TreeViewCancelEventArgs e)
        {
            if(e.Node is TableNode)
            {
                Table table = (Table)e.Node.Tag;

                if(GetInvalidColumns(table) == table.Columns.Count)
                {
                    e.Cancel = true;
                    return;
                }
            }
            else if(e.Node is ColumnNode)
            {
                if(!new ColumnValidator((Column)e.Node.Tag).IsValid())
                {
                    e.Cancel = true;
                    return;
                }
            }
        }

        private void trvDatabase_AfterCheck(object sender, TreeViewEventArgs e)
        {
            if(e.Action == TreeViewAction.ByMouse || e.Action == TreeViewAction.ByKeyboard)
            {
                if(e.Node is ColumnNode ||
                    e.Node is InputParameterNode ||
                    e.Node is InputOutputParameterNode ||
                    e.Node is OutputParameterNode)
                {
                    if(!NodesHelper.AreSiblingNodesChecked(e.Node))
                    {
                        e.Node.Parent.Checked = e.Node.Checked;
                    }
                }
                else
                {
                    NodesHelper.VerifyParent(e);

                    NodesHelper.CheckNodes(e.Node, e.Node.Checked);
                }
            }
            else
            {
                NodesHelper.VerifyParent(e);
            }
        }

        public override void OnActivated()
        {
            IDictionaryService dictservice = (IDictionaryService)GetService(typeof(IDictionaryService));
            ConnectionStringSettings connectionSettings = (ConnectionStringSettings)dictservice.GetValue("ConnectionSettings");

            if(dictservice.GetValue("TablesTreeView") == null)
            {
                //First Time
                RenderNodes();
                cnstring = connectionSettings.ConnectionString;
            }
            else
            {
                if(cnstring != connectionSettings.ConnectionString)
                {
                    //ConnectionString changed
                    RenderNodes();
                    cnstring = connectionSettings.ConnectionString;
                    dictservice.SetValue("TablesTreeView", null);
                    dictservice.SetValue("Tables", null);
                    dictservice.SetValue("Entities", null);
                }
            }

            base.OnActivated();
        }

        public override bool IsDataValid
        {
            get
            {
                bool response = false;

                if(this.tables != null)
                {
                    response = IsAtLeastOneNodeChecked();
                }

                return response;
            }
        }

        private void trvDatabase_Leave(object sender, EventArgs e)
        {
            SaveArgument();
        }

        private void trvDatabase_MouseLeave(object sender, EventArgs e)
        {
            SaveArgument();
        }
        #endregion

        #region Private Implementation
        private void RenderNodes()
        {
            Cursor.Current = Cursors.WaitCursor;

            try
            {
                IDictionaryService dictionary = (IDictionaryService)GetService(typeof(IDictionaryService));
                ConnectionStringSettings connectionSettings = (ConnectionStringSettings)dictionary.GetValue("ConnectionSettings");

                trvDatabase.Nodes.Clear();
                trvDatabase.Nodes.Add(new DataBaseNode(connectionSettings.Name));

                TreeNode tableNode = NodesHelper.GetTableNodes(connectionSettings, true);

                if(tableNode != null)
                {
                    trvDatabase.Nodes[0].Nodes.Add(tableNode);
                }

                TreeNode viewNode = NodesHelper.GetViewNodes(connectionSettings);
                if(viewNode != null)
                {
                    trvDatabase.Nodes[0].Nodes.Add(viewNode);
                }

                trvDatabase.Nodes[0].Expand();
            }
            catch
            {
                IUIService uiService = (IUIService)GetService(typeof(IUIService));

                uiService.ShowMessage(Microsoft.Practices.RepositoryFactory.Properties.Resources.ErrorObtainingMetadata);
            }

            Cursor.Current = Cursors.Default;
        }

        private bool IsAtLeastOneNodeChecked()
        {
            foreach(TreeNode node in trvDatabase.Nodes[0].Nodes)
            {
                if(node.Text == "Tables" || node.Text == "Views")
                {
                    foreach(TreeNode objectNode in node.Nodes)
                    {
                        if(objectNode.Checked)
                        {
                            return true;
                        }
                    }
                }
            }

            return false;
        }

        private ArrayList GetSelectedTables()
        {
            ArrayList selectedTables = new ArrayList();

            foreach(TreeNode node in trvDatabase.Nodes[0].Nodes)
            {
                if(node.Text == "Tables")
                {
                    foreach(TreeNode objectNode in node.Nodes)
                    {
                        if(objectNode.Checked)
                        {
                            selectedTables.Add((SD.Table)objectNode.Tag);
                        }
                    }
                }
            }

            return selectedTables;
        }

        private static int GetInvalidColumns(Table table)
        {
            ColumnValidator validator = new ColumnValidator();
            int invalidColumns = 0;

            foreach(Column column in table.Columns)
            {
                validator.Column = column;

                if(!validator.IsValid())
                {
                    invalidColumns++;
                }
            }

            return invalidColumns;
        }

        private void SaveArgument()
        {
            tables = trvDatabase;
            IDictionaryService dictservice = (IDictionaryService)GetService(typeof(IDictionaryService));
            dictservice.SetValue("TablesTreeView", tables);
            dictservice.SetValue("Tables", GetSelectedTables());
        }
        #endregion
    }
}