using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel.Design;
using EnvDTE;
using EnvDTE80;
using Microsoft.Practices.RepositoryFactory.ValueProviders;
using NUnit.Framework;
using EntityProperty = Microsoft.Practices.RepositoryFactory.CodeGeneration.ObjectModel.Property;
using ServiceContainer=Microsoft.Practices.ComponentModel.ServiceContainer;

namespace ValueProviderTests
{
    /// <summary>
    /// Summary description for MultipleClassesFixture
    /// </summary>
    [TestFixture]
    public class MultipleClassesFixture
    {
        private const string zipFileResourceName =
        "ValueProviderTests.SampleProjects.SingleProjectMultipleClasses.zip";
        private const string slnFileName = "SingleProjectMultipleClasses.sln";
        private const string projectName = "SingleProject";

        [Test]
        public void GetCorrectEntitiesWithMixedFullAndPartialClasses()
        {
            using( EntityValueProviderScaffold scaffold = new EntityValueProviderScaffold(zipFileResourceName, slnFileName, projectName ))
            {
                ArrayList entities = scaffold.GetEntities();
                GetExpectedEntities().AssertEntitesAreCorrect(entities);
            }
        }
        
        private static ExpectedEntitySet GetExpectedEntities()
        {
            return new ExpectedEntitySet(
                new ExpectedEntity( "NonPartialClass",
                    new ExpectedProperty( "Baz", typeof( float ) ),
                    new ExpectedProperty( "Bar", typeof( int ) ),
                    new ExpectedProperty( "Foo", typeof( string ) )
                ),
                new ExpectedEntity( "Account",
                    new ExpectedProperty( "Balance", typeof( decimal ) ),
                    new ExpectedProperty( "AccountNumber", typeof( string ) )
                ),                   
                new ExpectedEntity( "Customer",
                    new ExpectedProperty( "Phone", typeof( string ) ),
                    new ExpectedProperty( "Address1", typeof( string ) ),
                    new ExpectedProperty( "Address2", typeof(string ) ),
                    new ExpectedProperty( "City", typeof( string ) ),
                    new ExpectedProperty( "State", typeof( string ) ),
                    new ExpectedProperty( "Zip", typeof( int ) ),
                    new ExpectedProperty( "Name", typeof( string ) ),   
                    new ExpectedProperty( "Ssn", typeof( string ) )
                )    
            );
        }
    }
}
