using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;

using JavaIO = java.io;
using java.util.zip;
using System.CodeDom.Compiler;
using System.IO;
using java.util;

namespace ValueProvidersTests
{
    // A helper class to make it easier to suck
    // zip files out of embedded resources and
    // dump them onto the disk.
    class EmbeddedZipFile : IDisposable
    {
        private const int blockSize = 16384;
        private TempFileCollection tempDir = new TempFileCollection( );
        private string zipPath;
        private ZipFile zipFile;

        public EmbeddedZipFile(string resourceName)
            : this( Assembly.GetExecutingAssembly(), resourceName )
        {
        }

        public EmbeddedZipFile(Assembly assembly, string resourceName)
        {
            using (Stream embeddedZipStream = assembly.GetManifestResourceStream(resourceName))
            {
                zipPath = tempDir.AddExtension("zip", false);
                using (FileStream tempZipFile = new FileStream(zipPath, FileMode.Create))
                {
                    CopyStream(embeddedZipStream, tempZipFile, blockSize);
                }
            }

            zipFile = new ZipFile(zipPath);
        }

        public void ExtractTo(string path)
        {
            Enumeration e = zipFile.entries();
            while (e.hasMoreElements())
            {
                ZipEntry entry = (ZipEntry)e.nextElement();
                if( !entry.isDirectory())
                {
                    ExtractEntryTo(entry, path);
                }
            }
        }

        private void ExtractEntryTo(ZipEntry entry, string path)
        {
            string entryPath = Path.Combine( path, entry.getName());
            EnsurePathExists(entryPath);

            JavaIO.InputStream input = zipFile.getInputStream(entry);
            try
            {
                JavaIO.OutputStream output = new JavaIO.FileOutputStream(entryPath);
                try
                {
                    CopyStream(input, output, blockSize);
                }
                finally
                {
                    output.close();
                }
            }
            finally
            {
                input.close();
            }
        }

        private void EnsurePathExists(string path)
        {
            string directory = Path.GetDirectoryName(path);
            if (!Directory.Exists(directory))
            {
                Directory.CreateDirectory(directory);
            }
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (tempDir != null)
            {
                tempDir.Delete();
                tempDir = null;
            }
        }

        #endregion

        private void CopyStream(Stream input, Stream output, int blockSize)
        {
            byte[] buffer = new byte[blockSize];
            int numRead = input.Read(buffer, 0, blockSize);
            while (numRead > 0)
            {
                output.Write(buffer, 0, numRead);
                numRead = input.Read(buffer, 0, blockSize);
            }
        }

        private void CopyStream(JavaIO.InputStream input, JavaIO.OutputStream output, int blockSize)
        {
            sbyte[] buffer = new sbyte[blockSize];
            int numRead = input.read(buffer, 0, blockSize);
            while (numRead > 0)
            {
                output.write(buffer, 0, numRead);
                numRead = input.read(buffer, 0, blockSize);
            }
        }
    }
}
