using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Serialization;
using System.Globalization;
using Microsoft.Practices.Repository.Properties;

namespace Microsoft.Practices.Repository
{
    /// <summary>
    /// This exception is thrown when an unexpected error
    /// is received from the database.
    /// </summary>
    [Serializable]
    public class RepositoryFailureException : RepositoryException
    {
        /// <summary>
        /// Intializes an new instance of RepositoryFailureException.
        /// </summary>
        public RepositoryFailureException()
        {
        }

        /// <summary>
        /// Initializes a new instance of RepositoryFailureException.
        /// </summary>
        /// <param name="message">The message that describes the error.</param>
        public RepositoryFailureException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Initializes a new instance of RepositoryFailureException.
        /// </summary>
        /// <param name="message">The message that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception, or a null reference if no inner exception is specified.</param>
        public RepositoryFailureException(string message, Exception inner)
            : base(message, inner)
        {
        }

        /// <summary>
        /// Initializes a new instance of RepositoryFailureException.
        /// </summary>
        /// <param name="inner">The exception that is the cause of the current exception, or a null reference if no inner exception is specified.</param>
        public RepositoryFailureException(Exception inner)
            : base(string.Format(CultureInfo.CurrentCulture, Microsoft.Practices.Repository.Properties.GenericResources.DefaultMessage), inner)
        {
        }

        /// <summary>
        /// Initializes a new instance of RepositoryFailureException.
        /// </summary>
        /// <param name="info">The System.Runtime.Serialization.SerializationInfo that holds the serialized data
        /// about the exception being thrown.</param>
        /// <param name="context">The System.Runtime.Serialization.SerializationContext that contains contextual informationa about
        /// the source or destination.</param>
        protected RepositoryFailureException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }
    }
}
